package com.massa.mrules.demos.eshop.custom;

import java.io.IOException;
import java.util.Collection;

import com.massa.mrules.accessor.AbstractReadAccessor;
import com.massa.mrules.addon.IAddonsManager;
import com.massa.mrules.addon.MAddonsFinder;
import com.massa.mrules.context.IContext;
import com.massa.mrules.context.compile.ICompilationContext;
import com.massa.mrules.context.execute.IExecutionContext;
import com.massa.mrules.exception.MAccessorException;
import com.massa.mrules.exception.MConfigurationException;
import com.massa.mrules.exception.MRuleInternalException;
import com.massa.mrules.exception.MRuleValidationException;
import com.massa.mrules.util.CostInfo;
import com.massa.mrules.util.OuterWithLevel;

public class EshopAddonsManager implements IAddonsManager {

	public EshopAddonsManager() {
	}

	@Override
	public void registerAddons() throws MConfigurationException, MRuleInternalException {
		MAddonsFinder.registerAddon(new EshopAddonsManager.DiscountGenerator());
	}

	public static class DiscountGenerator extends AbstractReadAccessor {

		private static final long serialVersionUID = 1L;
		private static int idx = 0;

		@Override
		public void compileValue(ICompilationContext ctxt, Class<?> expectedType)
				throws MRuleValidationException {
		}

		@Override
		public <G, C extends Collection<G>> void compileCollection(
				ICompilationContext ctxt, Class<C> expectedType,
				Class<G> expectedGenericType) throws MRuleValidationException {
		}

		@Override
		public Object get(final IExecutionContext ctxt) throws MAccessorException {
			//Here should be placed the implementation of a real discount code generator.
			return "GENERATED_DISCOUNT" + (++idx);
		}

		@Override
		public String getImplementationId() {
			return "DISCOUNTGEN";
		}

		@Override
		public Class<?> getType(final IContext ctxt) throws MAccessorException {
			return String.class;
		}

		/**
		 * Returns false, as generator should always be called.
		 */
		@Override
		public boolean isCacheUsed() {
			return false;
		}

		@Override
		public CostInfo getEstimatedReadCost(ICompilationContext ctxt)
				throws MRuleValidationException {
			return new CostInfo(1);
		}

		@Override
		public boolean isShouldIterate() {
			return false;
		}

		@Override
		public Class<?> getGenericCollectionType(IContext ctxt)
				throws MAccessorException {
			return null;
		}

		@Override
		public void toString(OuterWithLevel outer) throws IOException {
			outer.write(this.getImplementationId());
		}

		@Override
		public boolean equals(final Object o) {
			return o != null && this.getClass() == o.getClass();
		}

		@Override
		public int hashCode() {
			return this.getClass().hashCode();
		}
	}
}
